package kr.co.vividnext.sodalive.live.room

import com.querydsl.core.types.OrderSpecifier
import com.querydsl.core.types.Predicate
import com.querydsl.core.types.dsl.CaseBuilder
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.domain.Pageable
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Repository
interface LiveRoomRepository : JpaRepository<LiveRoom, Long>, LiveRoomQueryRepository

interface LiveRoomQueryRepository {
    fun getLiveRoomList(
        dateString: String?,
        status: LiveRoomStatus,
        pageable: Pageable,
        member: Member,
        timezone: String,
        isAdult: Boolean
    ): List<LiveRoom>
}

class LiveRoomQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : LiveRoomQueryRepository {
    override fun getLiveRoomList(
        dateString: String?,
        status: LiveRoomStatus,
        pageable: Pageable,
        member: Member,
        timezone: String,
        isAdult: Boolean
    ): List<LiveRoom> {
        var where: Predicate

        if (status == LiveRoomStatus.NOW) {
            where = liveRoom.channelName.isNotNull
                .and(liveRoom.channelName.isNotEmpty)
        } else {
            where = if (dateString != null) {
                val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd")
                val date = LocalDate.parse(dateString, dateTimeFormatter).atStartOfDay()
                    .atZone(ZoneId.of(timezone))
                    .withZoneSameInstant(ZoneId.of("UTC"))
                    .toLocalDateTime()

                liveRoom.beginDateTime.goe(date)
                    .and(liveRoom.beginDateTime.lt(date.plusDays(1)))
                    .and(
                        liveRoom.channelName.isNull
                            .or(liveRoom.channelName.isEmpty)
                    )
            } else {
                liveRoom.beginDateTime.gt(
                    LocalDateTime.now()
                        .atZone(ZoneId.of(timezone))
                        .withZoneSameInstant(ZoneId.of("UTC"))
                        .toLocalDateTime()
                )
                    .and(
                        liveRoom.channelName.isNull
                            .or(liveRoom.channelName.isEmpty)
                    )
            }
        }

        if (!isAdult) {
            where = where.and(liveRoom.isAdult.isFalse)
        }

        where = where.and(liveRoom.isActive.isTrue)
            .and(liveRoom.member.isNotNull)
            .and(liveRoom.type.ne(LiveRoomType.SECRET))

        return queryFactory
            .selectFrom(liveRoom)
            .offset(pageable.offset)
            .limit(pageable.pageSize.toLong())
            .where(where)
            .orderBy(
                *orderByFieldAccountId(
                    memberId = member.id!!,
                    status = status,
                    offset = pageable.offset,
                    dateString = dateString
                )
            )
            .fetch()
    }

    private fun orderByFieldAccountId(
        memberId: Long,
        status: LiveRoomStatus,
        offset: Long,
        dateString: String?
    ): Array<out OrderSpecifier<*>> {
        return if (status == LiveRoomStatus.NOW) {
            arrayOf(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
        } else if (status == LiveRoomStatus.RESERVATION && offset == 0L && dateString == null) {
            arrayOf(
                CaseBuilder()
                    .`when`(member.id.eq(memberId)).then(1)
                    .otherwise(2)
                    .asc(),
                liveRoom.beginDateTime.asc()
            )
        } else {
            arrayOf(liveRoom.beginDateTime.asc())
        }
    }
}
