package kr.co.vividnext.sodalive.live.room

import kr.co.vividnext.sodalive.live.room.info.LiveRoomInfoRedisRepository
import kr.co.vividnext.sodalive.member.Member
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.domain.Pageable
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Service
class LiveRoomService(
    private val repository: LiveRoomRepository,
    private val roomInfoRepository: LiveRoomInfoRedisRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val coverImageHost: String
) {
    fun getRoomList(
        dateString: String?,
        status: LiveRoomStatus,
        pageable: Pageable,
        member: Member,
        timezone: String
    ): List<GetRoomListResponse> {
        return repository
            .getLiveRoomList(
                dateString = dateString,
                status = status,
                pageable = pageable,
                member = member,
                timezone = timezone,
                isAdult = member.auth != null
            )
            .asSequence()
            .map {
                val roomInfo = roomInfoRepository.findByIdOrNull(it.id!!)

                val beginDateTime = it.beginDateTime
                    .atZone(ZoneId.of("UTC"))
                    .withZoneSameInstant(ZoneId.of(timezone))

                GetRoomListResponse(
                    roomId = it.id!!,
                    title = it.title,
                    content = it.notice,
                    beginDateTime = beginDateTime.format(
                        DateTimeFormatter.ofPattern("yyyy.MM.dd E hh:mm a")
                    ),
                    numberOfParticipate = (roomInfo?.listenerCount ?: 0) +
                        (roomInfo?.speakerCount ?: 0) +
                        (roomInfo?.managerCount ?: 0),
                    numberOfPeople = it.numberOfPeople,
                    isAdult = it.isAdult,
                    price = it.price,
                    channelName = it.channelName,
                    managerNickname = it.member!!.nickname,
                    managerId = it.member!!.id!!,
                    tags = listOf(),
                    coverImageUrl = if (it.coverImage!!.startsWith("https://")) {
                        it.coverImage!!
                    } else {
                        "$coverImageHost/${it.coverImage!!}"
                    },
                    isReservation = false,
                    isPrivateRoom = it.type == LiveRoomType.PRIVATE
                )
            }
            .toList()
    }
}
